@@ -230,8 +230,10 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, const struct msa_module* info)
 
     xmlChar* temp = NULL;
     if (g_strcmp0((gchar *)function, GET_LIST_FRIENDS) == 0 || 
+        g_strcmp0((gchar *)function, UPDATE_WALL_MESSAGES) == 0 ||
         g_strcmp0((gchar *)function, UPDATE_INBOX_MESSAGES) == 0 ||
-        g_strcmp0((gchar *)function, UPDATE_OUTBOX_MESSAGES) == 0) {
+        g_strcmp0((gchar *)function, UPDATE_OUTBOX_MESSAGES) == 0 ||
+        g_strcmp0((gchar *)function, GET_PHOTOS) == 0) {
 
         node = xpath_get_node("//Request/Params/number[@name='from']", reqXml);
         if(node != NULL){
@@ -350,12 +352,17 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, struct ProfileData *cp)
     gchar* converted_friends = NULL;
     gchar* tmp = NULL;
     char* url;
+    char* file;
+    char* file_name;
+
     //gchar* class = get_req_class(reqXml);
     xmlChar* function = get_req_function(reqXml);
     int num = 0;
     int init_status = 0;
-    
-    url = make_url(reqXml, cp->remixmid, cp->remixsid);
+    xmlNodePtr node = NULL;
+    xmlNodePtr node2 = NULL;
+
+    url = make_url(reqXml, cp->remixmid, cp->remixsid, cp);
     
     if (url == NULL || strlen(url) < 1) {
         if (g_strcmp0((gchar *)function, GET_BINARY_DATA) == 0 ) {
@@ -369,7 +376,21 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, struct ProfileData *cp)
         }
     }
     int code = 0;
-    code = http_get(url, info_proxy, info_port, NULL, &tmp);
+    if (g_strcmp0((gchar *)function, UPLOAD_PHOTO) == 0 ) {
+        node = xpath_get_node("//Request/Params/string[@name='file']", reqXml);
+        node2 = xpath_get_node("//Request/Params/string[@name='fileName']", reqXml);
+        if (node != NULL && node2 != NULL){
+            file = xmlNodeGetContent(node);
+            file_name = xmlNodeGetContent(node2);
+            code = http_upload_photo(url, info_proxy, info_port, NULL, &tmp, cp, file, file_name);
+        } else {
+            info_message(DRV_REQ_ERROR_CODE, doc, DRV_REQ_ERROR, 1);
+            g_free(function);
+            return 0;
+        }
+    }
+    else
+        code = http_get(url, info_proxy, info_port, NULL, &tmp);
     if (code == 200) {
         if (g_strcmp0((gchar *)function, GET_BINARY_DATA) == 0 ) {
             g_free(function);
@@ -387,7 +408,8 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, struct ProfileData *cp)
         g_free(url);
         switch (check(tmp)) {
             case 0:
-                if (g_strcmp0((gchar *)function, SEND_MESSAGE) == 0) {
+                if (g_strcmp0((gchar *)function, SEND_MESSAGE) == 0 ||
+                    g_strcmp0((gchar *)function, SEND_WALL_MESSAGE) == 0) {
                     info_message(DRV_MESS_ERROR_CODE, doc, DRV_MESS_ERROR, 1);
 		    g_free(tmp);
                     xmlFree(function);
@@ -403,13 +425,13 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, struct ProfileData *cp)
 
                 if (g_strcmp0((gchar *)function, GET_MY_PROFILE) == 0 || 
                     g_strcmp0((gchar *)function, UPDATE_PROFILE) == 0) {
-                    if(parser_profile(doc, tmp, cp->remixmid, TRUE) == 1){
+                    if(parser_profile(doc, tmp, cp->remixmid, TRUE, cp) == 1){
                         info_message(DRV_RESP_ERROR_CODE, doc, DRV_RESP_ERROR, 1);
                     }
                     g_free(tmp);      	    
                 }
                 if (g_strcmp0((gchar *)function, GET_PROFILE_NO_IMAGE) == 0 ) {
-                    if(parser_profile(doc, tmp, cp->remixmid, FALSE) == 1){
+                    if(parser_profile(doc, tmp, cp->remixmid, FALSE, cp) == 1){
                         info_message(DRV_RESP_ERROR_CODE, doc, DRV_RESP_ERROR, 1);
                     }
                     g_free(tmp);      	    
@@ -422,6 +444,25 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, struct ProfileData *cp)
                     xmlFree(function);
                     return 0;
                 }
+                if (g_strcmp0((gchar *)function, GET_ALBUMS) == 0 ) {
+                    if(parser_profile_albums(doc, tmp, cp->remixmid, cp) == 1){
+                        info_message(DRV_RESP_ERROR_CODE, doc, DRV_RESP_ERROR, 1);
+                    }
+                    g_free(tmp);      	    
+                }
+                if (g_strcmp0((gchar *)function, GET_PHOTOS) == 0) {
+                    num = parser_photos(tmp, doc, counter);
+                    g_free(tmp);
+                    if (num == PAGE_SIZE && (counter + PAGE_SIZE) < max_count ) {
+                        counter += PAGE_SIZE;
+                        make_request( reqXml, doc, cp);
+                    } else {
+                        xmlFree(function);
+                        return 0;
+                    }
+                }
+
+
                 if (g_strcmp0((gchar *)function, GET_LIST_FRIENDS) == 0) {
                     g_debug("GET_LIST_FRIENDS");
                     converted_friends = g_convert(tmp, -1, UTF8, CP1251, NULL, NULL, NULL);
@@ -437,6 +478,7 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, struct ProfileData *cp)
                     }
                 }
                 if (g_strcmp0((gchar *)function, UPDATE_INBOX_MESSAGES) == 0 || 
+                    g_strcmp0((gchar *)function, UPDATE_WALL_MESSAGES) == 0 ||
                     g_strcmp0((gchar *)function, UPDATE_OUTBOX_MESSAGES) == 0) {
                     xmlChar* message_id = NULL;
                     xmlNodePtr node;
@@ -531,6 +573,14 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, struct ProfileData *cp)
 		break;
         }
 
+    } else if (code == 302){
+           if (g_strcmp0((gchar *)function, UPLOAD_PHOTO) == 0 ) {
+               if(tmp == NULL)
+                    info_message(DRV_PHOTO_UPLOAD_CODE, doc, DRV_PHOTO_UPLOAD, 0);
+           }
+        g_free(url);
+        g_free(function);
+        return 0;
     } else if (code >= 500){
         info_message(DRV_SERVER_ERROR_CODE, doc, DRV_SERVER_ERROR, 1);
         g_free(url);
@@ -841,11 +891,11 @@ int read_message(xmlDocPtr reqXml, char* remixmid, char* remixsid)
  * @param remixmid - authorization parameter
  * @return function of request.
  */
-gchar* make_url(xmlDocPtr reqXml, char* remixmid, char* remixsid)
+gchar* make_url(xmlDocPtr reqXml, char* remixmid, char* remixsid, struct ProfileData *cp)
 {
     g_debug("make_url START");
     xmlChar* url;
-    const char* params[11];
+    const char* params[13];
     xmlDocPtr urlXml;
     xmlNodePtr root;
     //xmlDocDump(stdout, reqXml);
@@ -858,12 +908,19 @@ gchar* make_url(xmlDocPtr reqXml, char* remixmid, char* remixsid)
     params[5] = g_strdup_printf("%d", counter);
     params[6] = g_strdup("MAXCOUNTER");
     params[7] = g_strdup_printf("%d", max_count);
+    params[8] = g_strdup("UPLOAD_URL");
+
+    if (cp->upload_url != NULL)
+        params[9] = g_strdup_printf("\"%s\"", cp->upload_url);
+    else
+        params[9] = g_strdup("url");
+
     if(captch == 1) {
-        params[8] = g_strdup("CAPTCHA");
-        params[9] = g_strdup("1");
-        params[10] = NULL;
+        params[10] = g_strdup("CAPTCHA");
+        params[11] = g_strdup("1");
+        params[12] = NULL;
     } else 
-        params[8] = NULL;
+        params[10] = NULL;
 
     xsltStylesheetPtr cur = NULL;
 
@@ -894,10 +951,12 @@ gchar* make_url(xmlDocPtr reqXml, char* remixmid, char* remixsid)
     g_free((gpointer*)params[5]);
     g_free((gpointer*)params[6]);
     g_free((gpointer*)params[7]);
+    g_free((gpointer*)params[8]);
+    g_free((gpointer*)params[9]);
 
     if(captch == 1) {
-        g_free((gpointer*)params[8]);
-        g_free((gpointer*)params[9]);
+        g_free((gpointer*)params[10]);
+        g_free((gpointer*)params[11]);
         captch = 0;
     }
     g_printf("%s", url);
@@ -1169,6 +1228,102 @@ int http_post(const char* url, const char* proxy, int port, char* post_data, cha
     return code;
 }
 
+int http_upload_photo(const char* url, const char* proxy, const int port, char** header, gchar** body, struct ProfileData *cp, char* file, char* file_name)
+{
+    CURL *curl;
+    CURLcode res;
+    long code;
+    struct MemoryStruct chunk,chunk2;
+
+    chunk.memory = NULL;
+    chunk.size = 0;
+
+    chunk2.memory = NULL;
+    chunk2.size = 0;
+
+    struct curl_httppost *formpost=NULL;
+    struct curl_httppost *lastptr=NULL;
+    struct curl_slist *headerlist=NULL;
+    static const char buf[] = "Expect:";
+
+    curl_global_init(CURL_GLOBAL_ALL);
+
+    curl_formadd(&formpost,
+                  &lastptr,
+         CURLFORM_COPYNAME, "aid",
+     CURLFORM_COPYCONTENTS, cp->aid,
+              CURLFORM_END);
+
+    curl_formadd(&formpost,
+                  &lastptr,
+         CURLFORM_COPYNAME, "hash",
+     CURLFORM_COPYCONTENTS, cp->hash,
+              CURLFORM_END);
+
+     curl_formadd(&formpost,
+                   &lastptr,
+          CURLFORM_COPYNAME, "rhash",
+      CURLFORM_COPYCONTENTS, cp->rhash,
+              CURLFORM_END);
+
+    /* Fill in the file upload field */
+    curl_formadd(&formpost,
+                  &lastptr,
+         CURLFORM_COPYNAME, "file1",
+             CURLFORM_FILE, file,
+         CURLFORM_FILENAME, file_name,
+      CURLFORM_CONTENTTYPE, "image/jpeg",
+              CURLFORM_END);
+
+    curl = curl_easy_init();
+    /* initalize custom header list (stating that Expect: 100-continue is not
+     *      wanted */
+    headerlist = curl_slist_append(headerlist, buf);
+
+    if(curl) {
+     /* what URL that receives this POST */
+        curl_easy_setopt(curl, CURLOPT_URL, url);
+
+     /* only disable 100-continue header if explicitly requested */
+        curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headerlist);
+        curl_easy_setopt(curl, CURLOPT_HTTPPOST, formpost);
+        curl_easy_setopt(curl, CURLOPT_PROXY, proxy);
+        curl_easy_setopt(curl, CURLOPT_PROXYPORT, port);
+        curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
+
+        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
+        curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION, WriteMemoryCallback);
+
+        curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *)&chunk);
+        curl_easy_setopt(curl, CURLOPT_WRITEHEADER, (void *)&chunk2);
+
+        res = curl_easy_perform(curl);
+
+        res = curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &code );
+        g_debug("CODE %ld", code);
+        g_debug("HEADER\n  %s", chunk2.memory);        
+        
+        if (header != NULL) {
+            *header = g_strdup(chunk2.memory);
+        }
+        
+        if(chunk2.memory)
+            g_free(chunk2.memory);
+        
+        if (body != NULL) {
+            *body = g_strdup(chunk.memory);
+        if(chunk.memory) 
+	        g_free(chunk.memory);
+        }
+        /* always cleanup */
+        curl_easy_cleanup(curl);
+        /* then cleanup the formpost chain */
+        curl_formfree(formpost);
+        /* free slist */
+        curl_slist_free_all (headerlist);
+    }
+   return code;
+}
 
 xmlNodePtr xpath_get_node(char* path, xmlDocPtr doc)
 {
