@@ -118,14 +118,15 @@ int parser_image(xmlDocPtr doc, char* data, char* url)
  *
  * @param doc - response
  * @param json - data in json format
+ * @param cp - current profile
  * @return 0 on success or 1 otherwise.
  */
-int parser_profile(xmlDocPtr doc, char* json, char* remixmid, gboolean image)
+int parser_profile(xmlDocPtr doc, char* json, char* remixmid, gboolean image, struct ProfileData *cp)
 {
     int i = 0, buf_serialized = 0;
     gchar* id = NULL;
     struct json_object* buf = NULL, *profile = NULL, *ht = NULL, *edu = NULL, *edu_item = NULL, *activity = NULL;
-    
+    struct json_object *ph = NULL, *hash = NULL, *rhash = NULL, *url = NULL, *aid = NULL;     
     xmlNodePtr rootNode, childNode, node, node2;
 
     rootNode = xpath_get_node("//Response/Params", doc);
@@ -195,6 +196,22 @@ int parser_profile(xmlDocPtr doc, char* json, char* remixmid, gboolean image)
         ht = json_object_object_get(profile, HABITATION_KEY);
         if(ht == NULL)
             return 1;
+
+        //PARSE PHOTO UPLOADS PARAMS
+        ph = json_object_object_get(profile, PH_KEY );
+
+        hash = json_object_object_get(ph, HASH_KEY );
+        cp->hash = g_strdup(json_object_get_string(hash));
+
+        rhash = json_object_object_get(ph, RHASH_KEY );
+        cp->rhash = g_strdup(json_object_get_string(rhash));
+
+        url = json_object_object_get(ph, URL_KEY );
+        cp->upload_url = g_strdup(json_object_get_string(url));
+
+        aid = json_object_object_get(ph, AID_KEY );
+        cp->aid = g_strdup(json_object_get_string(aid));
+
         //PARSE CITY
         childNode = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRUCT, NULL );
         xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "City");
@@ -375,6 +392,60 @@ int parser_profile(xmlDocPtr doc, char* json, char* remixmid, gboolean image)
     return 1;
 }
 
+
+/**
+ * Parses json data, form xml profile
+ *
+ * @param doc - response
+ * @param json - data in json format
+ * @param cp - current profile
+ * @return 0 on success or 1 otherwise.
+ */
+int parser_profile_albums(xmlDocPtr doc, char* json, char* remixmid, struct ProfileData *cp)
+{
+    struct json_object *profile = NULL;
+    struct json_object *ph = NULL, *hash = NULL, *rhash = NULL, *url = NULL, *aid = NULL;     
+    xmlNodePtr rootNode, childNode, node;
+
+    rootNode = xpath_get_node("//Response/Params", doc);
+	if(rootNode == NULL)
+        return 1;
+
+    profile = json_tokener_parse(json);
+    if(profile != NULL){
+
+        //PARSE PHOTO UPLOADS PARAMS
+        ph = json_object_object_get(profile, PH_KEY );
+
+        hash = json_object_object_get(ph, HASH_KEY );
+        cp->hash = g_strdup(json_object_get_string(hash));
+
+        rhash = json_object_object_get(ph, RHASH_KEY );
+        cp->rhash = g_strdup(json_object_get_string(rhash));
+
+        url = json_object_object_get(ph, URL_KEY );
+        cp->upload_url = g_strdup(json_object_get_string(url));
+
+        aid = json_object_object_get(ph, AID_KEY );
+        cp->aid = g_strdup(json_object_get_string(aid));
+
+        childNode = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_ARRAY, NULL );
+        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "albumList");
+        childNode = xmlAddChild(rootNode, childNode);
+
+        node = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRUCT, NULL );
+        xmlSetProp(node, BAD_CAST NAME, BAD_CAST "album");
+        xmlSetProp(node, BAD_CAST "id", BAD_CAST cp->aid);
+        node = xmlAddChild(childNode, node);
+	
+        json_object_put(profile);
+        g_debug("free profile");
+        return 0;
+    }
+
+    return 1;
+}
+
 /**
  * Parses json data, form xml friends list
  *
@@ -717,3 +788,91 @@ int parser_new_messages(char* json)
     }
     return 0;
 }
+
+
+/**
+ * Parses json data, form xml photos list
+ *
+ * @param doc - response
+ * @param json - data in json format
+ * @return number of photos
+ */
+int parser_photos(char *json, xmlDocPtr doc, int counter)
+{
+    int i=0;
+    int length = 0;
+    int photo_count = 0;
+    struct json_object *photo, *photos, *id, *url_small, *url_big;
+    char* tmp;
+    xmlNodePtr rootNode, childNode, node,node2;
+
+    rootNode = xpath_get_node("//Response/Params", doc);
+	if(rootNode == NULL)
+        return 0;
+    childNode = xpath_get_node("//Response/Params/array", doc);
+
+    if (childNode == NULL) {
+        node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_ARRAY,  NULL);
+        childNode = xmlAddChild(rootNode, node);
+        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "photosList");
+        tmp = g_strdup_printf("%d", counter);
+        xmlSetProp(childNode, BAD_CAST "from", BAD_CAST tmp);
+        xmlSetProp(childNode, BAD_CAST "to", BAD_CAST tmp);
+        g_free(tmp);
+    }
+
+    photos = json_tokener_parse(json);
+
+    length = json_object_array_length(photos);
+    for(i = 0; i < length; i++) {
+        photo = json_object_array_get_idx(photos, i );
+        
+        if (photo == NULL) {
+            length = i;
+            break;
+        }
+        id = json_object_array_get_idx(photo, 0 );
+        if (id == NULL) {
+            length = i;
+            break;
+        }
+        node = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRUCT, NULL );
+        xmlSetProp(node, BAD_CAST NAME, BAD_CAST "photo");
+        tmp = g_strconcat(PREFIX, json_object_get_string(id), NULL);
+        xmlSetProp(node, BAD_CAST "id", BAD_CAST tmp);
+        node = xmlAddChild(childNode, node);
+        free(tmp);
+
+        url_small = json_object_array_get_idx(photo, 1 );
+        if(url_small == NULL){
+            xmlUnlinkNode(node);
+            length = i;
+            break;
+        }
+        
+        node2 = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(url_small));
+        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "urlSmall");
+        node2 = xmlAddChild(node, node2);
+
+
+        url_big = json_object_array_get_idx(photo, 2 );
+        if(url_big == NULL){
+            xmlUnlinkNode(node);
+            length = i;
+            break;
+        }	
+        node2 = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(url_big));
+        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "urlBig");
+        node2 = xmlAddChild(node, node2);
+
+    }
+    json_object_put(photos);
+    xmlChar* len_str = xmlGetProp(childNode, BAD_CAST "to");
+    tmp = g_strdup_printf("%d", length + atoi((gchar *)len_str));
+    xmlSetProp(childNode, BAD_CAST "to", BAD_CAST tmp);
+
+    g_free(tmp);
+    xmlFree(len_str);
+
+    return length;
+}
