@@ -33,6 +33,10 @@ struct ProfileData {
     gchar *password;
     gchar *remixmid;
     gchar *remixsid;
+    gchar *hash;
+    gchar *rhash;
+    gchar *aid;
+    gchar *upload_url;
     struct ProfileData *next;
 };
 
